import os
import os.path
import shutil
from typing import List, Union
import logging

logger = logging.getLogger(__name__)

class Base:
    def __init__(self, storage_path: os.PathLike):
        self.storage_path = storage_path


class Downloader(Base):
    """ Downloader for OpenStack swift object storages
    """
    def download(self, remote_path: os.PathLike, local_dir: os.PathLike) -> os.PathLike:
        os.path.join(self.storage_path, remote_path)

        return shutil.copy2(os.path.join(self.storage_path, remote_path), local_dir)


class Uploader(Base):
    """ Uploader for OpenStack swift object storages
    """
    def upload(self, local_path: Union[os.PathLike, List[os.PathLike]], remote_dir: os.PathLike) -> List[os.PathLike]:
        paths = local_path if isinstance(local_path, List) else [local_path]
        remote_paths = [
            os.path.join(
                self.storage_path,
                remote_dir,
                os.path.basename(path)
            )
            for path in paths
        ]

        for local_path, remote_path in zip(paths, remote_paths):
            os.makedirs(os.path.dirname(remote_path), exist_ok=True)
            shutil.copy2(local_path, remote_path)

        return remote_paths

    def product_exists(self, remote_dir: os.PathLike) -> bool:
        remote_path = os.path.join(self.storage_path, remote_dir)
        for r, d, f in os.walk(remote_path):
            if len(f) >= 2:
                return True
        return False
